VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsWebState"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public Publishers As Collection

Private mSessionID As String

Private Sub Class_Initialize()
    Set Publishers = New Collection
End Sub

Public Property Get SessionID() As String
    SessionID = mSessionID
End Property
Public Property Let SessionID(ByVal lSessionID As String)
    mSessionID = lSessionID
    If mSessionID = "" Then ' Generate a new SessionID if none exists.
        Call GenerateSessionID
    ElseIf Len(mSessionID) <> 17 Or IsNumeric(mSessionID) = False Then ' Generate a new SessionID if invalid format.
        Call GenerateSessionID
    End If
End Property

' Generate a state SessionID by simply using the date and time and a random number.
' Scramble the parts to make it look more random.
Private Sub GenerateSessionID()
    Randomize
    mSessionID = Format(Now, "yyyymmddHhNnSs") & Format(Rnd * 10000, "00000")
End Sub

Private Property Get StateFileName() As String
    StateFileName = App.Path & "\" & mSessionID & ".dat"
End Property

' Saves all state information to a simple CSV file.
Public Sub SaveState()
    Dim lFileNbr   As Integer
    Dim lPubState  As clsPubState
    
    lFileNbr = FreeFile
    Open StateFileName For Output As lFileNbr
    
    ' Save the state data.
    Write #lFileNbr, CStr(Publishers.Count)
    For Each lPubState In Publishers
        Write #lFileNbr, lPubState.ID, lPubState.Name, lPubState.Phone
    Next lPubState
    
    Close lFileNbr
End Sub

' Attempts to restore the state information from the CSV file.
' If the state file is not found, or the data in the file hasn't
' been used in the last 60 minutes, then we assume a new session.
Public Sub GetState()
    Dim lFileNbr   As Integer
    Dim lNbrPubs   As Long
    Dim lIndex     As Long
    Dim lID        As Long
    Dim lName      As String
    Dim lPhone     As String
    Dim lPubState  As clsPubState
        
    On Error Resume Next
    lFileNbr = FreeFile
    Open StateFileName For Input As lFileNbr
    
    'If file not found, then that means no state information exists.
    If Err.Number = 53 Then ' 53 = File Not Found.
        Exit Sub
    ElseIf Err.Number <> 0 Then
        Call Err.Raise(Err.Number, Err.Source, Err.Description)
    End If
    
    On Error GoTo 0 ' Reset error handling so run-time errors get raised.
    
    ' If state information is > 60 minutes old, then ignore it.
    If DateDiff("n", FileDateTime(StateFileName), Now) > 60 Then
        Close lFileNbr
        Exit Sub
    End If
    
    ' Get the state data.
    Input #lFileNbr, lNbrPubs
    For lIndex = 1 To lNbrPubs Step 1
        Set lPubState = New clsPubState
        ' Note the syntax of the Input statement won't allow us to input
        ' directly into an object's properties so instead we have to input
        ' into local variables and then assign to the object.
        Input #lFileNbr, lID, lName, lPhone
        lPubState.ID = lID
        lPubState.Name = lName
        lPubState.Phone = lPhone
    Next lIndex
    
    Close lFileNbr
End Sub

